
//#define __AVR_LIBC_DEPRECATED_ENABLE__

#include <avr/io.h>
#include <avr/interrupt.h>

//#define LOWER_BOUND 0
//#define UPPER_BOUND 3

void waitloop(unsigned int n);

void wait100us()
{
	waitloop(100);
}

void wait600us()
{
	waitloop(600);
}

void set_led(int num, int val)
{
	PORTB = 0xff;
	switch (val) {
	case  0: PORTD = 0x3f; break;
	case  1: PORTD = 0x06; break;
	case  2: PORTD = 0x5b; break;
	case  3: PORTD = 0x4f; break;
	case  4: PORTD = 0x66; break;
	case  5: PORTD = 0x6d; break;
	case  6: PORTD = 0x7d; break;
	case  7: PORTD = 0x27; break;
	case  8: PORTD = 0x7f; break;
	case  9: PORTD = 0x6f; break;
	}
	PORTB = ~(1 << num);
}

void off_led()
{
	PORTB = 0xff;
}

unsigned char display_digits[4];
unsigned char reset_usisr_timeout;

//SIGNAL(SIG_OUTPUT_COMPARE0A)
//SIGNAL(OUTPUT_COMPARE0A_vect)
SIGNAL(TIMER0_COMPA_vect)
{
	TCNT0 = 0; 
	if (reset_usisr_timeout > 0) {
		reset_usisr_timeout--;
		if (reset_usisr_timeout == 0) {
			USISR = 0xf0;
		}
	}
}

//SIGNAL(SIG_USI_START)
SIGNAL(USI_START_vect)
{
	reset_usisr_timeout = 10;
	USISR |= 0x80;
}

//SIGNAL(SIG_USI_OVERFLOW)
SIGNAL(USI_OVERFLOW_vect)
{
	int c = USIDR;
	int n = c >> 4;
	if (n >= LOWER_BOUND && n <= UPPER_BOUND) {
		n -= LOWER_BOUND;
		display_digits[n & 3] = c & 0x0f;
	}
	reset_usisr_timeout = 0;
	USISR = 0xf0;
}

int main()
{
	TCCR0B = 0x03; // 1/64 prescale
	OCR0A = 125; // 1ms interval
	TIMSK |= 1 << OCIE0A;

	PORTB = 0x0f;
	PORTD = 0x00;
	DDRB = 0x0f;
	DDRD = 0xff;

	USICR = 0xd8;
	USISR = 0xf0;

	display_digits[0] = 0;
	display_digits[1] = 0;
	display_digits[2] = 0;
	display_digits[3] = 0;

	reset_usisr_timeout = 0;

	sei();

	while (1) {
		set_led(0, display_digits[0]);
		wait100us();
		set_led(1, display_digits[1]);
		wait100us();
		set_led(2, display_digits[2]);
		wait100us();
		set_led(3, display_digits[3]);
		wait100us();
		off_led();
		wait600us();
	}

	return 0;
}

