
#include "lcd.h"

unsigned char lcd_width;
unsigned char lcd_height;
unsigned char lcd_x;
unsigned char lcd_y;


void lcd_locate(int x, int y)
{
	lcd_x = x;
	lcd_y = y;
}

static void lcd_write_nibble(unsigned char c)
{
	lcd_set_d7(c & 0x08);
	lcd_set_d6(c & 0x04);
	lcd_set_d5(c & 0x02);
	lcd_set_d4(c & 0x01);
}

static void lcd_e_pulse()
{
	wait50us();
	lcd_set_e(0);
	wait50us();
	lcd_set_e(1);
}

static void lcd_write_byte(unsigned char c)
{
	lcd_write_nibble(c >> 4);
	lcd_e_pulse();
	lcd_write_nibble(c);
	lcd_e_pulse();
}

static void lcd_write_command(unsigned char c)
{
	lcd_set_rs(0);
	lcd_write_byte(c);
}

static void lcd_write_data(unsigned char c)
{
	lcd_set_rs(1);
	lcd_write_byte(c);
}

static unsigned char lcd_address()
{
	if (lcd_height == 2) {
		switch (lcd_y) {
		case 0:
			return 0x80 + lcd_x;
		case 1:
			return 0xc0 + lcd_x;
		}
	} else if (lcd_height == 4) {
		switch (lcd_y) {
		case 0:
			return 0x80 + lcd_x;
		case 1:
			return 0xc0 + lcd_x;
		case 2:
			return 0x94 + lcd_x;
		case 3:
			return 0xd4 + lcd_x;
		}
	}
	return 0;
}

void lcd_write_character(char c)
{
	unsigned char a = lcd_address();
	lcd_write_command(a);
	lcd_write_data((unsigned char)c);
}

void lcd_cls()
{
	lcd_write_command(0x01);
	wait_ms(2);
	lcd_locate(0, 0);
}

void lcd_newline()
{
	lcd_x = 0;
	lcd_y++;
	if (lcd_y >= lcd_height) {
		lcd_y = 0;
	}
}

void lcd_putc(char c)
{
	if (c == '\n') {
		lcd_newline();
	} else {
		lcd_write_character(c);
		lcd_x++;
		if (lcd_x >= lcd_width) {
			lcd_newline();
		}
	}
}

void lcd_print(char const *p)
{
#if 01
	while (*p) {
		lcd_putc(*p);
		p++;
	}
#else
	int i;
	for (i = 0; p[i]; i++) {
		lcd_putc(p[i]);
	}
#endif
}

void lcd_println(char const *p)
{
	lcd_print(p);
	lcd_putc('\n');
}

void lcd_init(int width, int height)
{
	int i;
	lcd_width = width;
	lcd_height = height;
	lcd_set_e(1);
	lcd_set_rs(0);
	wait_ms(15);
	for (i = 0; i < 3; i++) {
		lcd_write_byte(0x03);
		wait_ms(2);
	}
	lcd_write_byte(0x02);
	wait50us();
	lcd_write_command(0x28);
	lcd_write_command(0x0c);
	lcd_write_command(0x06);
	lcd_cls();
}

