
#include <avr/interrupt.h>
#include "system.h"

#define SCALE (CLOCK / 8 / 256 / 100)

static unsigned char _scale;
static volatile unsigned long _system_tick_count;
static volatile unsigned long _tick_count;
static volatile unsigned long _time_s;
static unsigned short _time_ms;

void system_init()
{
	_scale = 0;
	_tick_count = 0;
	_time_ms = 0;
	_time_s = 0;

	TCCR0B = 0x02; // 1/8 prescaling
	TIMSK0 |= 1 << TOIE0;
}

SIGNAL(TIMER0_OVF_vect)
{
	_system_tick_count++;
	_scale += 10;
	if (_scale >= SCALE) {
		_scale -= SCALE;
		_tick_count++;
		_time_ms++;
		if (_time_ms >= 1000) {
			_time_ms = 0;
			_time_s++;
		}
	}
}

unsigned long get_tick_count()
{
	return _tick_count;
}

void set_time(unsigned long s, unsigned short ms)
{
	_time_s = s;
	_time_ms = ms;
}

unsigned long get_time()
{
	return _time_s;
}

//

void sleep(unsigned long ms)
{
	unsigned long start_tick = _system_tick_count;
	unsigned long t = ms * SCALE / 10;
	while (_system_tick_count - start_tick < t);
}

//

void wait_us(unsigned int us)
{
	if (us > 0) {
		waitloop((unsigned short)(us * 5033UL / 2048UL)); // us * (CLOCK * 64 / 250000) / 2048
	}
}

void wait1ms()
{
#if 1
	waitloop(1000UL * (CLOCK / 800UL) / 10000UL);
#else
	wait_us(1000);
#endif
}

void wait_ms(unsigned int ms)
{
	int i;
	for (i = 0; i < ms; i++) {
		wait1ms();
	}
}

void wait50us()
{
	wait_us(50 * 196608 / 80000);
}
